import { getDownloadURL, ref, uploadBytesResumable } from "firebase/storage";
import React, { useState, useEffect, ChangeEvent, useRef } from "react";
import { useDispatch, useSelector } from "react-redux";
import { ACTIONS } from "../common/actions";
import { updateCandidate } from "../common/actions/user";
import { fileStorage } from "../common/config";
import { getProfile } from "../common/lib/util";
import { APP_STATE, CandidateType, OnboardSteps, USER_TYPE } from "../common/types";
import Avatar from "./Avatar";
import Progress from "./Progress";
import Onboarder from "./Onboarder";
import ProfilePicCrop from "./Modals/ProfilePicCrop";

const CandidateProfile = () => {
  const dispath = useDispatch();
  const candidate = useSelector(getProfile) as CandidateType;
  const [profile, setProfile] = useState<CandidateType>(candidate);
  const [resume, setResume] = useState<File | null>(null);
  const [selfIntro, setSelfIntro] = useState<File | null>(null);
  const [progress, setProgress] = useState<number>(0);
  const [onboardStep, setOnboardStep] = useState<OnboardSteps>(
    OnboardSteps.COMPLETE
  );
  const [profilePicPrev, setprofilePicPrev] = useState<string | null>(null);
  const _resumeInput = useRef<HTMLInputElement>(null);
  const _introInput = useRef<HTMLInputElement>(null);
  const _profilePicInput = useRef<HTMLInputElement>(null);

  useEffect(() => {
    if (candidate) {
      setProfile(candidate);
      checkOnboardingStatus(candidate);
    }
  }, [candidate]);

  const onChange = (e: ChangeEvent<HTMLInputElement>) => {
    if (e.target.name === "phone") {
      setProfile({
        ...profile,
        contacts: { ...profile.contacts, phone: e.target.value },
      });
    } else {
      setProfile({
        ...profile,
        [e.target.name]: e.target.value,
      });
    }
  };

  const onChangeAddress = (e: ChangeEvent<HTMLInputElement>) => {
    setProfile({
      ...profile,
      contacts: {
        ...profile.contacts,
        address: {
          ...profile.contacts.address,
          [e.target.name]: e.target.value,
        },
      },
    });
  };

  const onUpdateCandidate = (
    payload: Partial<CandidateType>,
    callBack?: (candidate: CandidateType) => void
  ) => {
    dispath(updateCandidate(payload, callBack));
  };

  const uploadSingleFile = (path: string, file: File) => {
    return new Promise((resolve, reject) => {
      const uploadRef = ref(fileStorage, path);
      const uploadTask = uploadBytesResumable(uploadRef, file);
      uploadTask.on(
        "state_changed",
        (snapshot) => {
          const percent = Math.round(
            (snapshot.bytesTransferred / snapshot.totalBytes) * 100
          );
          setProgress(percent);
        },
        (err) => {
          reject(err);
        },
        () => {
          getDownloadURL(uploadTask.snapshot.ref).then((url) => {
            setProgress(0);
            resolve(url);
          });
        }
      );
    });
  };

  const onSubmit = async () => {
    const { jobIds, state, _id, ...rest } = profile;
    dispath({
      type: ACTIONS.SET_APP_STATE,
      payload: { state: APP_STATE.LOADING },
    });
    if (resume || selfIntro) {
      const files = [];
      let update: any = {};
      if (resume) {
        files.push({
          path: `/resumes/${profile._id}.pdf`,
          file: resume,
          key: "resume",
        });
      }

      if (selfIntro) {
        files.push({
          path: `/selfIntros/${profile._id}.mp4`,
          file: selfIntro,
          key: "selfIntro",
        });
      }

      await Promise.all(
        files.map(async (_file) => {
          try {
            const url = await uploadSingleFile(_file.path, _file.file);
            update[_file.key] = url;
          } catch (error) {}
        })
      );

      onUpdateCandidate({ ...rest, ...update });
    } else {
      onUpdateCandidate(rest);
    }
  };

  const onSelectResume = (e: ChangeEvent<HTMLInputElement>) => {
    if (e.target.files) {
      setResume(e.target.files[0]);
    }
  };

  const onSelectSelfIntro = (e: ChangeEvent<HTMLInputElement>) => {
    if (e.target.files) {
      setSelfIntro(e.target.files[0]);
    }
  };

  const onSelectProfilePic = (e: ChangeEvent<HTMLInputElement>) => {
    if (e.target.files) {
      setprofilePicPrev(URL.createObjectURL(e.target.files[0]));
    }
  };

  const onPressOnboard = async () => {
    const update: any = {};
    if (onboardStep === OnboardSteps.RESUME && resume) {
      const url = await uploadSingleFile(`/resumes/${profile._id}.pdf`, resume);
      update.resume = url;
    }

    if (onboardStep === OnboardSteps.SELF_INTRO && selfIntro) {
      const url = await uploadSingleFile(
        `/selfIntros/${profile._id}.mp4`,
        selfIntro
      );
      update.selfIntro = url;
    }

    onUpdateCandidate(update, checkOnboardingStatus);
  };

  const checkOnboardingStatus = (_candidate: CandidateType) => {
    if (_candidate) {
      if (!_candidate?.resume) {
        setOnboardStep(OnboardSteps.RESUME);
      } else if (!_candidate?.selfIntro) {
        setOnboardStep(OnboardSteps.SELF_INTRO);
      } else {
        setOnboardStep(OnboardSteps.COMPLETE);
      }
    }
  };

  const onPressUploadProfilePic = () => {
    _profilePicInput?.current?.click();
  };

  const onPressUploadResume = () => {
    _resumeInput?.current?.click();
  };

  const onPressUploadSelfIntro = () => {
    _introInput?.current?.click();
  };

  const onRequestCloseProfilePic = () => {
    setprofilePicPrev(null);
  };

  const isOnboarded = onboardStep === OnboardSteps.COMPLETE;

  return (
    <div className="card p-4 mb-3">
      <h5>Edit/Update profile</h5>
      <div className="mb-3 row">
        <label className="col-sm-3 col-form-label">Name</label>
        <div className="col-sm-9">
          <input
            type="text"
            className="form-control"
            name="name"
            value={profile.name}
            onChange={onChange}
          />
        </div>
      </div>

      <div className="mb-3 row">
        <label className="col-sm-3 col-form-label">Address</label>
        <div className="col-sm-9">
          <input
            type="text"
            name="addressLine"
            className="form-control mb-2"
            placeholder="Address line"
            value={profile.contacts?.address?.addressLine}
            onChange={onChangeAddress}
          />
          <input
            type="text"
            className="form-control mb-2"
            placeholder="City"
            name="city"
            value={profile.contacts?.address?.city}
            onChange={onChangeAddress}
          />
          <input
            type="text"
            className="form-control mb-2"
            placeholder="Country"
            name="country"
            value={profile.contacts?.address?.country}
            onChange={onChangeAddress}
          />
        </div>
      </div>
      <div className="mb-3 row">
        <div className="col-sm-6">
          <div className="row">
            <label className="col-sm-4 col-form-label">Date of birth</label>
            <div className="col-sm-8">
              <input
                type="date"
                className="form-control"
                name="dateOfBirth"
                value={profile?.dateOfBirth}
                onChange={onChange}
              />
            </div>
          </div>
        </div>
        <div className="col-sm-6">
          <div className="row">
            <label className="col-sm-3 col-form-label">Phone</label>
            <div className="col-sm-9">
              <input
                type="text"
                className="form-control"
                name="phone"
                value={profile.contacts?.phone}
                onChange={onChange}
              />
            </div>
          </div>
        </div>
      </div>
      <div className="mb-3 row">
        <div className="col-sm-6">
          <div className="row">
            <label className="col-sm-4 col-form-label">Resume</label>
            <div className="col-sm-8">
              {profile?.resume && (
                <a
                  className="btn btn-secondary mb-2"
                  href={profile.resume}
                  target="_blank"
                  rel="noreferrer"
                >
                  Preview resume
                </a>
              )}
              <div className="d-md-block">
                <button
                  type="button"
                  className="btn btn-warning btn-sm"
                  style={{ width: "100%" }}
                  onClick={onPressUploadResume}
                >
                  Select Resume
                </button>
              </div>
            </div>
          </div>
        </div>
        <div className="col-sm-6">
          <div className="row">
            <label className="col-sm-4 col-form-label">Self Intro</label>

            <div className="col-sm-8">
              {profile?.selfIntro && (
                <a
                  className="btn btn-secondary mb-2"
                  href={profile.selfIntro}
                  target="_blank"
                  rel="noreferrer"
                >
                  Preview video
                </a>
              )}
              <div className="d-md-block">
                <button
                  type="button"
                  className="btn btn-warning btn-sm"
                  style={{ width: "100%" }}
                  onClick={onPressUploadSelfIntro}
                >
                  Select Video
                </button>
              </div>
            </div>
          </div>
        </div>
      </div>
      <div className="mb-3 row">
        <div className="col-sm-6">
          <div className="row">
            <div className="col-sm-4 ">
              <label className="col-form-label">Profile picuture</label>
            </div>
            <div className="col-sm-8">
              <Avatar
                name={profile?.name}
                url={profile?.profilePicture}
                size="lg"
              />
              <button
                type="button"
                className="btn btn-link"
                style={{ padding: "0" }}
                onClick={onPressUploadProfilePic}
              >
                Upload picture
              </button>
            </div>
          </div>
        </div>
      </div>

      <div className="d-md-block">
        <button
          type="button"
          className="btn btn-primary"
          style={{ width: "300px", float: "right" }}
          onClick={onSubmit}
        >
          Update Profile
        </button>
      </div>
      <Progress progress={progress} />

      <input
        type="file"
        className="form-control hidden-input"
        onChange={onSelectResume}
        accept="application/pdf"
        ref={_resumeInput}
      />
      <input
        type="file"
        className="form-control hidden-input"
        accept="video/mp4"
        onChange={onSelectSelfIntro}
        ref={_introInput}
      />
      <input
        type="file"
        className="form-control hidden-input"
        accept="image/*"
        onChange={onSelectProfilePic}
        ref={_profilePicInput}
      />

      <Onboarder
        isOpen={!isOnboarded}
        onboardStep={onboardStep}
        onPressUploadResume={onPressUploadResume}
        onPressUploadSelfIntro={onPressUploadSelfIntro}
        progress={progress}
        onPressOnboard={onPressOnboard}
      />
      <ProfilePicCrop
        onRequestClose={onRequestCloseProfilePic}
        profilePicPrev={profilePicPrev}
        userId={profile._id}
        onUpdateCandidate={onUpdateCandidate}
        userType={USER_TYPE.CANDIDATE}
      />
    </div>
  );
};

export default CandidateProfile;
