import React, { useEffect, useMemo, useRef, useState } from "react";
import {
  MeetingProvider,
  useMeeting,
  useParticipant,
} from "@videosdk.live/react-sdk";
import ReactPlayer from "react-player";
import { meetingToken } from "../common/apis/meetings";

function ParticipantView({ participantId }: { participantId: string }) {
  const micRef = useRef<any>();
  const { webcamStream, micStream, webcamOn, micOn, isLocal } =
    useParticipant(participantId);

  const videoStream = useMemo(() => {
    if (webcamOn && webcamStream) {
      const mediaStream = new MediaStream();
      mediaStream.addTrack(webcamStream.track);
      return mediaStream;
    }
  }, [webcamStream, webcamOn]);

  useEffect(() => {
    if (micRef.current) {
      if (micOn && micStream) {
        const mediaStream = new MediaStream();
        mediaStream.addTrack(micStream.track);

        micRef.current.srcObject = mediaStream;
        micRef.current
          .play()
          .catch((error: any) =>
            console.error("videoElem.current.play() failed", error)
          );
      } else {
        micRef.current.srcObject = null;
      }
    }
  }, [micStream, micOn]);

  return (
    <div>
      <audio ref={micRef} autoPlay playsInline muted={isLocal} />
      {webcamOn && (
        <div className="player-wrapper">
          <ReactPlayer
            playsinline // very very imp prop
            pip={false}
            light={false}
            controls={false}
            muted={true}
            playing={true}
            url={videoStream}
            width="100%"
            height="100%"
            onError={(err) => {
              console.log(err, "participant video error");
            }}
            className="react-player"
          />
        </div>
      )}
    </div>
  );
}

function MeetingView({
  onMeetingLeave,
  record,
}: {
  onMeetingLeave: () => void;
  record?: boolean;
}) {
  const [joined, setJoined] = useState<string | null>(null);
  //Get the method which will be used to join the meeting.
  //We will also get the participants list to display all participants
  const {
    join,
    participants,
    leave,
    toggleMic,
    toggleWebcam,
    startRecording,
    localMicOn,
    localWebcamOn,
    isRecording,
    stopRecording,
  } = useMeeting({
    //callback for when meeting is joined successfully
    onMeetingJoined: () => {
      setJoined("JOINED");
    },
    onMeetingLeft: () => {
      if (record) {
        stopRecording();
      }
      onMeetingLeave();
    },
  });

  const joinMeeting = () => {
    setJoined("JOINING");
    join();
  };

  const onToggleMic = () => {
    toggleMic();
  };

  const onToggleCamera = () => {
    toggleWebcam();
  };

  const onToggleRecord = () => {
    if (isRecording) {
      stopRecording();
    } else {
      startRecording();
    }
  };

  return (
    <div className="meeting-container">
      {joined && joined === "JOINED" ? (
        <>
          <div className="action-containers">
            <button className="btn btn-danger" onClick={leave}>
              Leave
            </button>
            <button className="meeting-action" onClick={onToggleMic}>
              {!localMicOn ? (
                <i className="fa-solid fa-microphone-slash"></i>
              ) : (
                <i className="fa-solid fa-microphone"></i>
              )}
            </button>
            <button className="meeting-action" onClick={onToggleCamera}>
              {!localWebcamOn ? (
                <i className="fa-solid fa-video-slash"></i>
              ) : (
                <i className="fa-solid fa-video"></i>
              )}
            </button>
            {record && (
              <button
                className="meeting-action record"
                onClick={onToggleRecord}
              >
                {isRecording ? (
                  <i className="fa-solid fa-stop"></i>
                ) : (
                  <i className="fa-solid fa-record-vinyl"></i>
                )}
              </button>
            )}
          </div>

          {[...participants.keys()].map((participantId) => (
            <ParticipantView
              participantId={participantId}
              key={participantId}
            />
          ))}
        </>
      ) : joined && joined === "JOINING" ? (
        <div className="lobby-actions">
          <p>Joining the meeting...</p>
        </div>
      ) : (
        <div className="lobby-actions">
          <button
            type="button"
            className="btn btn-primary"
            onClick={joinMeeting}
          >
            Join the meeting
          </button>
        </div>
      )}
    </div>
  );
}

const Meeting = ({
  roomId,
  onMeetingLeave,
  record = true,
}: {
  roomId: string;
  onMeetingLeave: () => void;
  record?: boolean;
}) => {
  if (!meetingToken) {
    return <div>Session ended</div>;
  }

  return (
    <MeetingProvider
      config={{
        meetingId: roomId,
        micEnabled: true,
        webcamEnabled: true,
        name: "Kavi's Org",
      }}
      token={meetingToken}
    >
      <MeetingView onMeetingLeave={onMeetingLeave} record={record} />
    </MeetingProvider>
  );
};

export default Meeting;
