import React, {
  useState,
  useMemo,
  useEffect,
  useRef,
  ChangeEvent,
} from "react";
import { useDispatch, useSelector } from "react-redux";
import { applyForJob } from "../common/actions/user";
import { getProfile, getStatusColor, getUserId } from "../common/lib/util";
import {
  CandidateType,
  JobPayloadType,
  JobType,
  OrganizationType,
  Reducers,
  USER_TYPE,
} from "../common/types";
import Jobs from "../components/Jobs";
import Layout from "../Layouts/Layout";
import CreateUpdateJob from "../components/Modals/CreateUpdateJob";
import Profile from "../components/Profile";
import { setDialogBox } from "../common/actions/common";
import CommonAPI from "../common/apis/common";
import { useNavigate } from "react-router-dom";

const CandidateHome = ({ jobs }: { jobs: JobPayloadType[] }) => {
  const dispatch = useDispatch();
  const navigate = useNavigate();
  const candidate = useSelector(getProfile) as CandidateType;
  const [selectedJob, setSelectedJob] = useState<number>(0);
  const userId = useSelector(getUserId);

  const job = jobs.length > 0 ? jobs[selectedJob] : null;

  const renderSkills = (skill: string, index: number) => (
    <span className="skill display" key={index}>
      {skill}
    </span>
  );

  const alreadyApplied = useMemo(() => {
    const foundCandidate = jobs[selectedJob]?.applications?.findIndex(
      (_item) => _item.candidate === userId
    );

    if (foundCandidate !== undefined && foundCandidate >= 0) {
      const status = jobs[selectedJob]?.applications?.[foundCandidate].status;
      const interview =
        jobs[selectedJob]?.applications?.[foundCandidate].interview;
      return {
        applied: true,
        status,
        color: getStatusColor(status),
        interview,
      };
    } else {
      return { applied: false, status: "", interview: null };
    }
  }, [jobs, selectedJob, userId]);

  const onApply = () => {
    if (!candidate?.resume || !candidate?.selfIntro) {
      dispatch(
        setDialogBox("Your resume and self introduction video is required!")
      );
    } else {
      dispatch(applyForJob(jobs[selectedJob]._id || ""));
    }
  };

  const onClickJoin = () => {
    navigate(
      `/meeting?roomId=${alreadyApplied.interview?.link}&jobId=${jobs[selectedJob]._id}`
    );
  };

  return (
    <div className="row">
      <div className="col-3 ">
        <Profile
          name={candidate.name}
          email={candidate.contacts.email}
          profilePic={candidate.profilePicture}
          address={candidate.contacts.address}
        />
      </div>
      <div className="col">
        <Jobs
          jobs={jobs}
          onSelectJob={setSelectedJob}
          selectedIndex={selectedJob}
        />
      </div>
      <div className="col">
        {job && (
          <div className="card p-4 job-preview">
            <h5>{job.title}</h5>
            <label>
              <a href={`/organizations?id=${job.organization[0]._id}`}>
                By {job.organization[0].name}
              </a>
            </label>
            <p>
              Salary : {job.salary.min} - {job.salary.max} {job.salary.currency}{" "}
            </p>
            <p className="desc">{job.description}</p>
            <label>Primary skills</label>
            <div className="skills">{job.primarySkills.map(renderSkills)}</div>
            <label>Secondary skills</label>
            <div className="skills">
              {job.secondarySkills.map(renderSkills)}
            </div>
            {alreadyApplied.applied ? (
              <>
                <p className="h5 mt-2">
                  Status :{" "}
                  <label className={alreadyApplied.color}>
                    {alreadyApplied.status}
                  </label>
                </p>

                {alreadyApplied?.interview?.date && (
                  <>
                    <p className="h6">
                      Interview is scheduled on :{" "}
                      <label>
                        {`${alreadyApplied.interview.date} at ${alreadyApplied.interview.time}`}
                      </label>
                    </p>
                    {alreadyApplied.interview?.link &&
                    alreadyApplied.interview?.link !== "ended" ? (
                      <button
                        className="btn btn-success mt-3"
                        onClick={onClickJoin}
                      >
                        Join now
                      </button>
                    ) : null}
                  </>
                )}
              </>
            ) : (
              <button className="btn btn-success mt-3" onClick={onApply}>
                Apply
              </button>
            )}
          </div>
        )}
      </div>
    </div>
  );
};

const OrganizationHome = () => {
  const [selectedJob, setSelectedJob] = useState<number>(0);
  const organization: OrganizationType = useSelector(
    (state: Reducers) => state.auth.organization
  ) as OrganizationType;
  const jobs = useSelector((state: Reducers) => state.common.jobs) as JobType[];
  const job = jobs.length > 0 ? jobs[selectedJob] : null;

  const renderSkills = (skill: string, index: number) => (
    <span className="skill" key={index}>
      {skill}
    </span>
  );

  return (
    <div className="row">
      <div className="col-3 ">
        <Profile
          name={organization.name}
          profilePic={organization.profilePicture}
          email={organization.contacts.email}
          address={organization.contacts.address}
        />
        <CreateUpdateJob mode="create" />
      </div>
      <div className="col">
        <Jobs
          jobs={jobs}
          onSelectJob={setSelectedJob}
          selectedIndex={selectedJob}
        />
      </div>
      <div className="col">
        {job && (
          <div className="card p-4 job-preview">
            <h5>{job.title}</h5>
            <p>
              Salary : {job.salary.min} - {job.salary.max} {job.salary.currency}{" "}
            </p>
            <p className="desc">{job.description}</p>
            <label>Primary skills</label>
            <div className="skills">{job.primarySkills.map(renderSkills)}</div>
            <label>Secondary skills</label>
            <div className="skills">
              {job.secondarySkills.map(renderSkills)}
            </div>
            <div className="mt-2">
              <label>Applicattions : </label>{" "}
              <a href={`/applications?jobId=${job._id}`}>
                ({job.applications?.length}) Candidates
              </a>
            </div>
            <CreateUpdateJob defaultJob={job} mode="update" />
          </div>
        )}
      </div>
    </div>
  );
};

const Home = () => {
  const [selectedJobs, setSelectedJobs] = useState<JobPayloadType[]>([]);
  const userType = useSelector((state: Reducers) => state.auth.userType);
  const _searchKey = useRef<string>("");
  const jobs = useSelector(
    (state: Reducers) => state.common.jobs
  ) as JobPayloadType[];

  useEffect(() => {
    setSelectedJobs(jobs);
  }, [jobs]);

  const onChangeSearchKey = (e: ChangeEvent<HTMLInputElement>) => {
    _searchKey.current = e.target.value;
  };

  const onClickSearch = () => {
    if (_searchKey.current !== "") {
      CommonAPI.searchJobs(_searchKey.current).then(({ data }: any) => {
        setSelectedJobs(data);
      });
    }
  };

  const renderSuffix =
    userType === USER_TYPE.CANDIDATE ? (
      <div className="input-group mb-3">
        <input
          type="text"
          className="form-control"
          placeholder="Search job"
          onChange={onChangeSearchKey}
        />
        <button
          className="btn btn-secondary"
          type="button"
          onClick={onClickSearch}
        >
          Search
        </button>
      </div>
    ) : undefined;

  return (
    <Layout title="Vacancies and Jobs" suffix={renderSuffix}>
      {userType === USER_TYPE.ORGANIZATION ? (
        <OrganizationHome />
      ) : (
        <CandidateHome jobs={selectedJobs} />
      )}
    </Layout>
  );
};

export default Home;
