import React, { useState, useEffect, useMemo } from "react";
import { useDispatch, useSelector } from "react-redux";
import {
  CandidateType,
  JobPayloadType,
  OrganizationType,
  Reducers,
} from "../common/types";
import Jobs from "../components/Jobs";
import Layout from "../Layouts/Layout";
import Profile from "../components/Profile";
import { useSearchParams } from "react-router-dom";
import { getProfile, getStatusColor, getUserId } from "../common/lib/util";
import { setDialogBox } from "../common/actions/common";
import { applyForJob } from "../common/actions/user";

const OrganizationHome = ({
  organization,
  jobs,
}: {
  organization: OrganizationType;
  jobs: JobPayloadType[];
}) => {
  const dispatch = useDispatch();
  const candidate = useSelector(getProfile) as CandidateType;
  const [selectedJob, setSelectedJob] = useState<number>(0);
  const job = jobs.length > 0 ? jobs[selectedJob] : null;
  const userId = useSelector(getUserId);

  const renderSkills = (skill: string, index: number) => (
    <span className="skill" key={index}>
      {skill}
    </span>
  );

  const alreadyApplied = useMemo(() => {
    const foundCandidate = jobs[selectedJob]?.applications?.findIndex(
      (_item) => _item.candidate === userId
    );
    if (foundCandidate !== undefined && foundCandidate >= 0) {
      const status = jobs[selectedJob]?.applications?.[foundCandidate].status;
      return {
        applied: true,
        status,
        color: getStatusColor(status),
      };
    } else {
      return { applied: false, status: "" };
    }
  }, [jobs, selectedJob, userId]);

  const onApply = () => {
    if (!candidate?.resume || !candidate?.selfIntro) {
      dispatch(
        setDialogBox("Your resume and self introduction video is required!")
      );
    } else {
      dispatch(applyForJob(jobs[selectedJob]._id || ""));
    }
  };

  return (
    <div className="row">
      <div className="col-3 ">
        <Profile
          name={organization.name}
          profilePic={organization.profilePicture}
          email={organization.contacts.email}
          address={organization.contacts.address}
          editProfile={false}
        />
      </div>
      <div className="col">
        <Jobs
          jobs={jobs}
          onSelectJob={setSelectedJob}
          selectedIndex={selectedJob}
        />
      </div>
      <div className="col">
        {job && (
          <div className="card p-4 job-preview">
            <h5>{job.title}</h5>
            <p>
              Salary : {job.salary.min} - {job.salary.max} {job.salary.currency}{" "}
            </p>
            <p className="desc">{job.description}</p>
            <label>Primary skills</label>
            <div className="skills">{job.primarySkills.map(renderSkills)}</div>
            <label>Secondary skills</label>
            <div className="skills">
              {job.secondarySkills.map(renderSkills)}
            </div>
            {alreadyApplied.applied ? (
              <p className="h5 mt-2">
                Status :{" "}
                <label className={alreadyApplied.color}>
                  {alreadyApplied.status}
                </label>
              </p>
            ) : (
              <button className="btn btn-success mt-3" onClick={onApply}>
                Apply
              </button>
            )}
          </div>
        )}
      </div>
    </div>
  );
};

const Organization = () => {
  const [selectedOrg, setSelectedOrg] = useState<OrganizationType>();
  const [selectedJobs, setSelectedJobs] = useState<JobPayloadType[]>([]);
  const [searchParams] = useSearchParams();
  const jobs = useSelector((state: Reducers) => state.common.jobs) as any[];

  useEffect(() => {
    const orgId = searchParams.get("id");
    const foundjobs = jobs.filter((_job) => _job.organization[0]._id === orgId);
    setSelectedOrg(foundjobs[0].organization[0]);
    setSelectedJobs(foundjobs);
  }, [jobs, searchParams]);

  return (
    <Layout title={selectedOrg?.name || "Searching.."}>
      {selectedOrg && (
        <OrganizationHome organization={selectedOrg} jobs={selectedJobs} />
      )}
    </Layout>
  );
};

export default Organization;
